from django.core.cache import cache
from django.contrib.auth.decorators import login_required
from django.http import HttpResponseRedirect, HttpResponse, Http404
from django.views.decorators.http import require_http_methods
from django.shortcuts import redirect, render_to_response
from django.db.models import Max
from django.template import RequestContext
from django_oauth_client.decorators import oauth_token_required
from django_oauth_client.utils import get_client
from django.forms import ModelForm
from u1to.uploader.models import Uploader
from django.contrib.admin import widgets
from django.template.defaultfilters import slugify
from django.conf import settings

import base64, json, urllib, datetime
from u1to.utils import u1url

class UploaderForm(ModelForm):
    class Meta:
        model = Uploader
        exclude = ('user', 'slug')

@login_required
@oauth_token_required(settings.U1_SERVER)
@require_http_methods(['GET', 'POST'])
def index(request):
    uploaders = []
    for u in Uploader.objects.filter(user=request.user):
        d = {
            "name": u.name,
            "slug": u.slug,
            "full_url": request.build_absolute_uri(u.get_url()),
            "id": u.id,
            "expires": u.expires
        }
        if u.password:
            d["protected"] = "Yes"
        else:
            d["protected"] = "No"
        uploaders.append(d)
    if request.method == "GET":
        form = UploaderForm()
        return render_to_response(
            'create-upload.html', {
                "form": form,
                "uploaders": uploaders
            },
            context_instance=RequestContext(request))
    else:
        instance = Uploader(user=request.user)
        form = UploaderForm(request.POST, instance=instance)
        if form.is_valid():
            form.save(commit=False)
            instance.slug = slugify(instance.name)
            instance.save()
            return HttpResponseRedirect("/upload")
        return render_to_response(
            'create-upload.html', {
                "form": form,
                "uploaders": uploaders
            },
            context_instance=RequestContext(request))

@login_required
@oauth_token_required(settings.U1_SERVER)
@require_http_methods(['POST'])
def remove(request):
    try:
        uploader = Uploader.objects.get(
            id=request.POST.get("id", None), user=request.user)
    except Uploader.DoesNotExist:
        raise Http404
    uploader.delete()
    return HttpResponseRedirect("/upload")
    


@require_http_methods(['GET', 'POST'])
def uploader(request, username, slug):
    try:
        uploader = Uploader.objects.get(
            slug=slug, user__username=username)
    except Uploader.DoesNotExist:
        raise Http404
    if uploader.expires and datetime.datetime.now() > uploader.expires:
        return HttpResponse("Expired")
    if uploader.password:
        # we do basic auth
        authok = False
        auth = request.META.get('HTTP_AUTHORIZATION',"").split()
        if len(auth) == 2:
            if auth[0].lower() == "basic":
                uname, passwd = base64.b64decode(auth[1]).split(':')
                if uname == username and passwd == uploader.password:
                    authok = True
        if not authok:
            response = HttpResponse("Password required.")
            response.status_code = 401
            response['WWW-Authenticate'] = 'Basic realm="Login required (enter username as %s)"' % username
            return response
    successful = []
    failed = []
    for key, file in request.FILES.items():
        client = get_client(uploader.user, settings.U1_SERVER)
        resp, content = client.request(
            u1url("file_api_base", "/api/file_storage/v1/~/Ubuntu%20One/u1to"))
        if resp['status'] == '404':
            resp, content = client.request(
                u1url("/api/file_storage/v1/~/Ubuntu%20One/u1to"),
                method="PUT",
                body=json.dumps({"kind": "directory"})
            )
        resp, content = client.request(
            u1url("file_api_base", "/api/file_storage/v1/~/Ubuntu%%20One/u1to/%s" % slug))
        if resp['status'] == '404':
            resp, content = client.request(
                u1url("file_api_base", "/api/file_storage/v1/~/Ubuntu%%20One/u1to/%s" % slug),
                method="PUT",
                body=json.dumps({"kind": "directory"})
            )
        out = json.loads(content)
        content_path = out["content_path"]
        file_content_path = "%s/%s" % (urllib.quote(content_path, safe="/~"), 
            urllib.quote(file.name, safe="/"))
        full_content_path = u1url("content_url_base", file_content_path)
        resp, content = client.request(
            full_content_path,
            method="PUT",
            body=file.read()
        )
        try:
            out = json.loads(content)
        except ValueError:
            failed.append({"name": file.name, "reason": "Ubuntu One failed on upload"})
            continue
        if out.get("error"):
            failed.append({"name": file.name, "reason": out.get("error")})
        else:
            successful.append(file.name)
    if request.META.get("HTTP_X_REQUESTED_WITH", "") == "XMLHttpRequest":
        return HttpResponse(json.dumps({"successful": successful, "failed": failed}))
    return render_to_response("uploader.html", { 
            "uploader": uploader,
            "successful": successful,
            "failed": failed
        }, context_instance=RequestContext(request))

